var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { fromHex, toHex } from '../utils';
var KEY_BYTE_SIZE = {
    128: 16,
    192: 24,
    256: 32
};
var KEY_BIT_SIZE = {
    16: 128,
    24: 192,
    32: 256
};
var MODES = {
    GCM: 'AES-GCM',
    CBC: 'AES-CBC'
};
var IVS = {
    GCM: crypto.getRandomValues(new Uint8Array(12)),
    CBC: crypto.getRandomValues(new Uint8Array(16))
};
function validateKeyAndMode(mode, rawKey, bitSize) {
    if (!Object.prototype.hasOwnProperty.call(KEY_BIT_SIZE, rawKey.length)) {
        throw new Error("invalid key AES key length, key length should be 16, 24 or 32 bytes");
    }
    if (rawKey.length !== KEY_BYTE_SIZE[bitSize]) {
        throw new Error("invalid key length, AES ".concat(bitSize, " key length should be ").concat(KEY_BYTE_SIZE[bitSize], " bytes length"));
    }
    if (!Object.prototype.hasOwnProperty.call(MODES, mode)) {
        throw new Error("invalid mode, mode ".concat(mode, " does not exist"));
    }
}
function importKey(rawKey, mode) {
    return __awaiter(this, void 0, void 0, function () {
        var encoder;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    encoder = new TextEncoder();
                    return [4 /*yield*/, crypto.subtle.importKey('raw', encoder.encode(rawKey), {
                            name: MODES[mode],
                            length: KEY_BIT_SIZE[rawKey.length]
                        }, true, ['encrypt', 'decrypt'])];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function parseEncryptedData(encryptedData, mode) {
    var ivs = {
        GCM: fromHex(encryptedData.slice(0, 24)),
        CBC: fromHex(encryptedData.slice(0, 32))
    };
    var cipherDatas = {
        GCM: fromHex(encryptedData.slice(24, encryptedData.length)),
        CBC: fromHex(encryptedData.slice(32, encryptedData.length))
    };
    return { iv: ivs[mode], cipherData: cipherDatas[mode] };
}
function encrypt(key, mode, data) {
    return __awaiter(this, void 0, void 0, function () {
        var importedKey, encoder, encoded, iv, alg, cipherData, cipherDataBuf, allBuf;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, importKey(key, mode)];
                case 1:
                    importedKey = _a.sent();
                    encoder = new TextEncoder();
                    encoded = encoder.encode(data);
                    iv = IVS[mode];
                    alg = {
                        name: MODES[mode],
                        iv: iv
                    };
                    return [4 /*yield*/, crypto.subtle.encrypt(alg, importedKey, encoded)];
                case 2:
                    cipherData = _a.sent();
                    cipherDataBuf = new Uint8Array(cipherData);
                    allBuf = new Uint8Array(iv.length + cipherDataBuf.length);
                    allBuf.set(iv);
                    allBuf.set(cipherDataBuf, iv.length);
                    return [2 /*return*/, toHex(allBuf)];
            }
        });
    });
}
function decrypt(key, mode, encryptedData) {
    return __awaiter(this, void 0, void 0, function () {
        var importedKey, parsedEncryptedData, alg;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, importKey(key, mode)];
                case 1:
                    importedKey = _a.sent();
                    parsedEncryptedData = parseEncryptedData(encryptedData, mode);
                    alg = {
                        name: MODES[mode],
                        iv: parsedEncryptedData.iv
                    };
                    return [4 /*yield*/, crypto.subtle.decrypt(alg, importedKey, parsedEncryptedData.cipherData)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
// CBC Encrypt
export function encryptWithAes128Cbc(key, data) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 128);
    return encrypt(key, mode, data);
}
// not yet supported
export function encryptWithAes192Cbc(key, data) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 192);
    return encrypt(key, mode, data);
}
export function encryptWithAes256Cbc(key, data) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 256);
    return encrypt(key, mode, data);
}
// CBC Decrypt
export function decryptWithAes128Cbc(key, encryptedData) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 128);
    return decrypt(key, mode, encryptedData);
}
// not yet supported
export function decryptWithAes192Cbc(key, encryptedData) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 192);
    return decrypt(key, mode, encryptedData);
}
export function decryptWithAes256Cbc(key, encryptedData) {
    var mode = 'CBC';
    validateKeyAndMode(mode, key, 256);
    return decrypt(key, mode, encryptedData);
}
// GCM Encrypt
export function encryptWithAes128Gcm(key, data) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 128);
    return encrypt(key, mode, data);
}
// not yet supported
export function encryptWithAes192Gcm(key, data) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 192);
    return encrypt(key, mode, data);
}
export function encryptWithAes256Gcm(key, data) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 256);
    return encrypt(key, mode, data);
}
// GCM Decrypt
export function decryptWithAes128Gcm(key, encryptedData) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 128);
    return decrypt(key, mode, encryptedData);
}
// not yet supported
export function decryptWithAes192Gcm(key, encryptedData) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 192);
    return decrypt(key, mode, encryptedData);
}
export function decryptWithAes256Gcm(key, encryptedData) {
    var mode = 'GCM';
    validateKeyAndMode(mode, key, 256);
    return decrypt(key, mode, encryptedData);
}
