var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { str2ab, DIGEST, DIGEST_SALT_LENGTH } from '../utils';
function loadPublicKey(pem) {
    var pemHeader = '-----BEGIN PUBLIC KEY-----';
    var pemFooter = '-----END PUBLIC KEY-----\n';
    var pemContents = pem.substring(pemHeader.length, pem.length - pemFooter.length);
    // base64 decode the string to get the binary data
    var binaryDerString = window.atob(pemContents);
    // convert from a binary string to an ArrayBuffer
    var binaryDer = str2ab(binaryDerString);
    return binaryDer;
}
function loadPrivateKey(pem) {
    // fetch the part of the PEM string between header and footer
    var pemHeader = '-----BEGIN PRIVATE KEY-----';
    var pemFooter = '-----END PRIVATE KEY-----\n';
    var pemContents = pem.substring(pemHeader.length, pem.length - pemFooter.length);
    // base64 decode the string to get the binary data
    var binaryDerString = window.atob(pemContents);
    // convert from a binary string to an ArrayBuffer
    var binaryDer = str2ab(binaryDerString);
    return binaryDer;
}
function loadPublicKeyForEncryption(pem, alg) {
    return __awaiter(this, void 0, void 0, function () {
        var binaryDer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    binaryDer = loadPublicKey(pem);
                    return [4 /*yield*/, crypto.subtle.importKey('spki', binaryDer, {
                            name: 'RSA-OAEP',
                            hash: alg
                        }, true, ['encrypt'])];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function loadPublicKeyForVerifySignature(pem, alg) {
    return __awaiter(this, void 0, void 0, function () {
        var binaryDer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    binaryDer = loadPublicKey(pem);
                    return [4 /*yield*/, crypto.subtle.importKey('spki', binaryDer, {
                            name: 'RSA-PSS',
                            hash: alg
                        }, true, ['verify'])];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
//  format should be PKCS8
function loadPrivateKeyForEncryption(pem, alg) {
    return __awaiter(this, void 0, void 0, function () {
        var binaryDer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    binaryDer = loadPrivateKey(pem);
                    return [4 /*yield*/, window.crypto.subtle.importKey('pkcs8', binaryDer, {
                            name: 'RSA-OAEP',
                            hash: alg
                        }, true, ['decrypt'])];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
//  format should be PKCS8
function loadPrivateKeyForSigning(pem, alg) {
    return __awaiter(this, void 0, void 0, function () {
        var binaryDer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    binaryDer = loadPrivateKey(pem);
                    return [4 /*yield*/, window.crypto.subtle.importKey('pkcs8', binaryDer, {
                            name: 'RSA-PSS',
                            hash: alg
                        }, true, ['sign'])];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function encryptWithOAEP(publicKey, hash, plainData) {
    return __awaiter(this, void 0, void 0, function () {
        var key;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, loadPublicKeyForEncryption(publicKey, hash)];
                case 1:
                    key = _a.sent();
                    return [4 /*yield*/, window.crypto.subtle.encrypt({
                            name: 'RSA-OAEP'
                        }, key, plainData)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function decryptWithOAEP(privateKey, hash, encryptedData) {
    return __awaiter(this, void 0, void 0, function () {
        var key;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, loadPrivateKeyForEncryption(privateKey, hash)];
                case 1:
                    key = _a.sent();
                    return [4 /*yield*/, window.crypto.subtle.decrypt({
                            name: 'RSA-OAEP'
                        }, key, encryptedData)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
// encryption
export function encryptWithOAEPSha1(publicKey, plainData) {
    return encryptWithOAEP(publicKey, DIGEST['SHA-1'], plainData);
}
export function encryptWithOAEPSha256(publicKey, plainData) {
    return encryptWithOAEP(publicKey, DIGEST['SHA-256'], plainData);
}
export function encryptWithOAEPSha384(publicKey, plainData) {
    return encryptWithOAEP(publicKey, DIGEST['SHA-384'], plainData);
}
export function encryptWithOAEPSha512(publicKey, plainData) {
    return encryptWithOAEP(publicKey, DIGEST['SHA-512'], plainData);
}
// decryption
export function decryptWithOAEPSha1(privateKey, encryptedData) {
    return decryptWithOAEP(privateKey, DIGEST['SHA-1'], encryptedData);
}
export function decryptWithOAEPSha256(privateKey, encryptedData) {
    return decryptWithOAEP(privateKey, DIGEST['SHA-256'], encryptedData);
}
export function decryptWithOAEPSha384(privateKey, encryptedData) {
    return decryptWithOAEP(privateKey, DIGEST['SHA-384'], encryptedData);
}
export function decryptWithOAEPSha512(privateKey, encryptedData) {
    return decryptWithOAEP(privateKey, DIGEST['SHA-512'], encryptedData);
}
// digital signature
// sign data with PSS and return a promise a promise that fulfills an array buffer containing the signature
function signWithPss(privateKey, hash, data) {
    return __awaiter(this, void 0, void 0, function () {
        var key;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, loadPrivateKeyForSigning(privateKey, hash)];
                case 1:
                    key = _a.sent();
                    return [4 /*yield*/, window.crypto.subtle.sign({
                            name: 'RSA-PSS',
                            saltLength: DIGEST_SALT_LENGTH[hash]
                        }, key, data)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
// verify signature with PSS and return a Promise that fulfills with a boolean value: true if the signature is valid, false otherwise.
function verifySignatureWithPss(publicKey, signature, hash, data) {
    return __awaiter(this, void 0, void 0, function () {
        var key;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, loadPublicKeyForVerifySignature(publicKey, hash)];
                case 1:
                    key = _a.sent();
                    return [4 /*yield*/, window.crypto.subtle.verify({
                            name: 'RSA-PSS',
                            saltLength: DIGEST_SALT_LENGTH[hash]
                        }, key, signature, data)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
export function signWithPssSha1(privateKey, data) {
    return signWithPss(privateKey, DIGEST['SHA-1'], data);
}
export function signWithPssSha256(privateKey, data) {
    return signWithPss(privateKey, DIGEST['SHA-256'], data);
}
export function signWithPssSha384(privateKey, data) {
    return signWithPss(privateKey, DIGEST['SHA-384'], data);
}
export function signWithPssSha512(privateKey, data) {
    return signWithPss(privateKey, DIGEST['SHA-512'], data);
}
export function verifySignatureWithPssSha1(publicKey, signature, data) {
    return verifySignatureWithPss(publicKey, signature, DIGEST['SHA-1'], data);
}
export function verifySignatureWithPssSha256(publicKey, signature, data) {
    return verifySignatureWithPss(publicKey, signature, DIGEST['SHA-256'], data);
}
export function verifySignatureWithPssSha384(publicKey, signature, data) {
    return verifySignatureWithPss(publicKey, signature, DIGEST['SHA-384'], data);
}
export function verifySignatureWithPssSha512(publicKey, signature, data) {
    return verifySignatureWithPss(publicKey, signature, DIGEST['SHA-512'], data);
}
